# camera_stats.rb - WebDialog to view/set camera specs.
# Copyright 2010, Martin Rinehart

require 'Sketchup'

module Camera_stats

$model = Sketchup.active_model
$view = $model.active_view
$cam = $view.camera
$observing = false
$observer = nil

class Camera_statsObserver < Sketchup::ViewObserver
    def initialize( wd )
        @wd = wd
    end
    def onViewChanged( view )
        if $observing
            script = 'get_data( "' + Camera_stats::get_json() + '" );'
            @wd.execute_script( script )
        end
    end
end # of class Camera_statsObserver

def self.get_json()
    json = '{'
    json += ' eye:[' +
        two_decimals( $cam.eye().x() ) + ', ' +
        two_decimals( $cam.eye().y() ) + ', ' +
        two_decimals( $cam.eye().z() ) + '], '
    json += ' target:[' +
        two_decimals( $cam.target().x() ) + ', ' +
        two_decimals( $cam.target().y() ) + ', ' +
        two_decimals( $cam.target().z() ) + '], '
    json += ' up:[' +
        two_decimals( $cam.up().x() ) + ', ' +
        two_decimals( $cam.up().y() ) + ', ' +
        two_decimals( $cam.up().z() ) + '], '
    json += ' orthographic:' + ( (!$cam.perspective?( )).to_s() ) + ', '
    json += ' direction:[' +
        two_decimals( $cam.direction().x() ) + ', ' +
        two_decimals( $cam.direction().y() ) + ', ' +
        two_decimals( $cam.direction().z() ) + '], '
    if ( $cam.perspective?() ) 
        json += ' fov:' + two_decimals( $cam.fov() ) + ', ' +
            ' focal_length:' + $cam.focal_length().to_i().to_s() + ', '
    else 
        json += ' height:' + two_decimals( $cam.height() ) + ', '
    end
    json += ' aspect_ratio:' + $cam.aspect_ratio().to_s() + ', '
    json += " description: '" + $cam.description() + "'"
    json += '}'
    
    return json
    
end # get_json()

def self.make_arr( str )
    arr = str.split( ',' )
    for i in 0..2 do arr[i] = arr[i].to_f() end
    return arr
end # of make_arr()

def self.two_decimals( length )
    len = length.to_inch().to_s()
    
    # bos, optional "-", any number of digits, a period, two more digits
    /^(-?\d*\.\d\d)/.match( len )
    return $1 if $1
    
    # bos, optional "-", any number of digits, a period, one more digit
    /^(-?\d*\.\d)/.match( len )
    return $1 if $1
    
    # bos, optional "-", any number of digits
    /^(-?\d*)/.match( len )
    return $1 if $1
    
    raise 'two_decimals could not handle ' + len
end # two_decimals()

wd = UI::WebDialog.new( "Camera Statistics", true, "camera_statistics",
    536, 243, 100, 100, true )
pn = File.expand_path( File.dirname(__FILE__) )
pn = File.join( pn, "camera_stats.html" )
wd.set_file( pn )

wd.add_action_callback( "close" ) { | dlg, msg | wd.close() }
wd.add_action_callback( "get_data" ) { | dlg, msg |
    script = 'get_data( "' + get_json() + '" );'
    wd.execute_script( script )
}

wd.add_action_callback( "new_data" ) { | dlg, msg |
    begin
        val = wd.get_element_value( 'eye' )
        eye = make_arr( val )
    rescue 
        raise 'Could not interpret "' + val + '" for eye.'
    end
    begin
        val = wd.get_element_value( 'target' )
        target = make_arr( val )
    rescue 
        raise 'Could not interpret "' + val + '" for target.'
    end
    begin
        val = wd.get_element_value( 'up' )
        up = make_arr( val )
    rescue 
        raise 'Could not interpret "' + val + '" for up.'
    end
    $cam.set( eye, target, up )
    $cam.perspective = msg == 'false' # msg == ortho check box status
    if $cam.perspective?()
        val = wd.get_element_value( 'field_of_view' ).to_f()
        val = 35 if ( val < 1 ) || ( val > 120 )
        $cam.fov = val
        # resetting fov changes focal length
    else
        val = wd.get_element_value( 'height' ).to_f()
        val = 100 if val <= 0.0
        $cam.height = val
    end
    $cam.aspect_ratio = wd.get_element_value( 'aspect_ratio' ).to_f()
    $cam.description = wd.get_element_value( 'descrip' )
    
    # this because changing eye/target changes direction and up
    # changing field of view changes focal length
    script = 'get_data( "' + get_json() + '" );'
    wd.execute_script( script )


} # end of new_data() ac

wd.add_action_callback( "observe" ) {
    if $observing
        $observing = false
        $view.remove_observer( $observer )
    else
        $observing = true
        $observer = Camera_statsObserver.new( wd )
        $view.add_observer( $observer )
    end
}

wd.show()

end # module Camera_stats

# end of camera_stats.rb